
/*  gBackGround changes your root window background.
    Copyright (C) 1999  Govind Salinas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef GBACKGROUND_H
  #include "gBackGround.h"
#endif
#ifndef PATH_MAX
  #define PATH_MAX 255
#endif

GtkWidget *filesel, *preview, *entry, *pbox;
GtkWidget *dclist, *fclist, *adclist, *afclist;
GList *ftemp, *dtemp;
gchar *dselected, *fselected;
gchar cwd[PATH_MAX + 1];
gint selected = 0;

void activate_entry(GtkWidget *widget, gpointer data)
{
  struct stat statbuf;
  gchar *name;
  const gchar *mime;
 
  name = strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
  if (stat(name, &statbuf) == -1) {
    fprintf(stderr, "%s:", name);  fflush(stderr);
    perror("Could not get stats on file!!");
    g_free(name);
    return;
  }
  if (S_ISDIR(statbuf.st_mode)) {
    gtk_clist_clear(GTK_CLIST(fclist));
    gtk_clist_clear(GTK_CLIST(dclist));
    get_dir_info(name);
    g_free(name);
    return;
  }
  else if (((userealmime && (mime = gnome_mime_type_of_file(name))) ||
          (mime = gnome_mime_type(name))) &&
          !g_strncasecmp("image", mime, 5)) {
    show_preview(NULL, (gpointer)name);
    g_free(name);
    return;
  }
}

void add_files_or_directories(GtkWidget *widget, gpointer data)
{
  struct stat statbuf;
  gchar *name[] = {""};
  const gchar *mime;
  
  name[0] = strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
  if (stat(name[0], &statbuf) == -1) {
    fprintf(stderr, "%s:", name[0]);  fflush(stderr);
    perror("Could not get stats on file!!");
    g_free(name[0]);
    return;
  }
  if (S_ISDIR(statbuf.st_mode)) {
    dtemp = g_list_append(dtemp, name[0]);
    gtk_clist_append(GTK_CLIST(adclist), name);
  }
  else if (((userealmime && (mime = gnome_mime_type_of_file(name[0]))) ||
          (mime = gnome_mime_type(name[0]))) &&
          !g_strncasecmp("image", mime, 5)) {
    ftemp = g_list_append(ftemp, name[0]);
    gtk_clist_append(GTK_CLIST(afclist), name);
  }
}

void remove_files_or_directories(GtkWidget *widget, gpointer data)
{
  gchar *name[] = {""};
  GList *li, *holdit = NULL;
  gint done = 0;

  if (selected == 1) {
    selected = 0;
    gtk_clist_clear(GTK_CLIST(adclist));
    gtk_clist_freeze(GTK_CLIST(adclist));
    for (li = g_list_nth(dtemp, 1); li != NULL; li = g_list_next(li)) {
      if (!strcmp(dselected, (name[0] = (gchar *)li->data)) && !done) {
        holdit = li;
        done = 1;
      }
      else gtk_clist_append(GTK_CLIST(adclist), name);
    }
    if (holdit) {
      dtemp = g_list_remove_link(dtemp, holdit);
      g_list_free(holdit);
    }
    gtk_clist_thaw(GTK_CLIST(adclist));
  }
  if (selected == 2) {
    selected = 0;
    gtk_clist_clear(GTK_CLIST(afclist));
    gtk_clist_freeze(GTK_CLIST(afclist));
    for (li = g_list_first(ftemp); li != NULL; li = g_list_next(li)) {
      if (!strcmp(fselected, (name[0] = (gchar *)li->data)) && !done) {
        holdit = li;
        done = 1;
      }
      else gtk_clist_append(GTK_CLIST(afclist), name);
    }
    if (holdit) {
      ftemp = g_list_remove_link(ftemp, holdit);
      g_list_free(holdit);
    }
    gtk_clist_thaw(GTK_CLIST(afclist));
  }
}

void show_preview(GtkWidget *widget, gpointer data)
{
  GdkImlibImage *image;
  gint w, h;
  gfloat scale;

  if (GTK_IS_WIDGET(preview)) gtk_widget_destroy(preview);
  image = gdk_imlib_load_image((gchar *)data);
  if (image) {
    h = image->rgb_height;
    w = image->rgb_width;
    if (h > w) scale = ((float)150/h);
    else scale = ((float)150/w);
    h = scale*h;
    w = scale*w;
    preview = gnome_pixmap_new_from_imlib_at_size(image, w, h);
    gdk_imlib_kill_image(image);
    gtk_box_pack_start(GTK_BOX(pbox), preview, TRUE, FALSE, 5);
    gtk_widget_show(preview);
  }
}

void kill_sel(GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy(filesel);
  filesel = NULL;
}

void update(GtkWidget *widget, gpointer data)
{
  GList *cur;
  gchar **dnames, **fnames;
  gint length1, length2, l;

  dnames = (gchar **)g_malloc(sizeof(gchar *) *
                            ((length1 = g_list_length(dtemp)) + 1));
  for (l = 1; l < length1; l++)
    dnames[l - 1] = strdup((gchar *)(g_list_nth(dtemp, l))->data);
  fnames = (gchar **)g_malloc(sizeof(gchar *) *
                            ((length2 = g_list_length(ftemp)) + 1));
  for (l = 0; l < length2; l++)
    fnames[l] = strdup((gchar *)(g_list_nth(ftemp, l))->data);

  gtk_widget_destroy(filesel);
  filesel = NULL;
  (g_list_first(dirnames))->data = NULL;
  gnome_uri_list_free_strings(dirnames);
  gnome_uri_list_free_strings(filelist);
  g_list_free(indfiles);
  for (cur = g_list_first(dirlist); cur != NULL; cur = g_list_next(cur))
    g_list_free(cur->data);
  g_list_free(dirlist);
  dirlist = indfiles = dirnames = filelist = NULL;
  dirlist = makedlist(dirlist, dnames, length1 - 1);
  indfiles = makeflist(indfiles, fnames, length2);
  if (window) gtk_combo_set_popdown_strings(GTK_COMBO(combo), dirnames);
  for (l = 0; l < length1 - 1; l++) g_free(dnames[l]);
  for (l = 0; l < length2; l++) g_free(fnames[l]);
}

void select_row(GtkWidget *widget, gint row, gint column,
                GdkEventButton *bevent, gpointer data)
{
  gchar *temp, *temp2, *enttext;

  if (data == dclist) {
    gtk_clist_get_text(GTK_CLIST(dclist), row, column, &temp);
    temp2 = strdup(temp);
    gtk_clist_clear(GTK_CLIST(fclist));
    gtk_clist_clear(GTK_CLIST(dclist));
    get_dir_info(temp2);
    g_free(temp2);
    return;
  }
  if (data == fclist) {
    gtk_clist_get_text(GTK_CLIST(fclist), row, column, &temp);
    enttext = (gchar *)g_malloc(sizeof(gchar) * (strlen(temp) +
                                strlen(cwd) + 2));
    strcpy(enttext, cwd);
    if (enttext[strlen(enttext)] != '/') strcat(enttext, "/");
    strcat(enttext, temp);
    gtk_entry_set_text(GTK_ENTRY(entry), enttext);
    show_preview(NULL, (gpointer)temp);
    return;
  }
  if (data == adclist) {
    selected = 1;
    gtk_clist_unselect_all(GTK_CLIST(afclist));
    gtk_clist_get_text(GTK_CLIST(adclist), row, column, &temp);
    dselected = strdup(temp);
    return;
  } 
  if (data == afclist) {
    selected = 2;
    gtk_clist_unselect_all(GTK_CLIST(adclist));
    gtk_clist_get_text(GTK_CLIST(afclist), row, column, &temp);
    fselected = strdup(temp);
    show_preview(NULL, (gpointer)fselected);
  } 
}

void get_dir_info(gchar *dirname)
{

  DIR *dir;
  struct dirent *dent;
  struct stat statbuf;
  gchar *name[] = {""}, *filename;
  const gchar *mime;

  if ((dir = opendir(dirname)) == NULL) {
    perror("Could not open directory!!");
    g_free(dirname);
    return;
  }
  chdir(dirname);
  if (getcwd(cwd, PATH_MAX) == NULL) {
    perror("Could not get current working directory!!");
    return;
  } 
  gtk_clist_freeze(GTK_CLIST(dclist));
  gtk_clist_freeze(GTK_CLIST(fclist));
  while ((dent = readdir(dir)) != NULL) {
    filename = (char *)g_malloc(sizeof(char) * (strlen(dent->d_name) + 1));
    strcpy(filename, dent->d_name);
    if (stat(filename, &statbuf) == -1) {
      fprintf(stderr, "%s:", filename);  fflush(stderr);
      perror("Could not get stats on file!!");
      g_free(filename);
      continue;
    }
    if (S_ISDIR(statbuf.st_mode)) {
      name[0] = filename;
      gtk_clist_append(GTK_CLIST(dclist), name);
    }
    else if (((userealmime && (mime = gnome_mime_type_of_file(filename))) ||
           (mime = gnome_mime_type(filename))) &&
           !g_strncasecmp("image", mime, 5)) {
      name[0] = filename;
      gtk_clist_append(GTK_CLIST(fclist), name);
    }
  }   
  gtk_clist_thaw(GTK_CLIST(dclist));
  gtk_clist_thaw(GTK_CLIST(fclist));
  gtk_entry_set_text(GTK_ENTRY(entry), strdup(cwd));
}

void add_items(GtkWidget *widget, gpointer data)
{
  GtkWidget *box, *button, *box2, *label, *arrow, *box3, *scroll, *frame;
  GList *li;
  gchar *name[] = {""};

  if (filesel) return;
  filesel = gnome_dialog_new("Add and Remove Files or Directories.",
                             GNOME_STOCK_BUTTON_OK,
                             GNOME_STOCK_BUTTON_CANCEL, NULL);
  gtk_container_set_border_width(GTK_CONTAINER(filesel), 0);
  gnome_dialog_button_connect((GnomeDialog *)filesel, 1, kill_sel,
                               (gpointer) filesel);
  gnome_dialog_button_connect((GnomeDialog *)filesel, 0, update, NULL);
  gtk_signal_connect(GTK_OBJECT(filesel), "delete_event",
                     GTK_SIGNAL_FUNC(kill_sel),  (gpointer)filesel);
  gtk_signal_connect(GTK_OBJECT(filesel), "destroy",
                     GTK_SIGNAL_FUNC(kill_sel), (gpointer)filesel);
  box = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(filesel)->vbox), box, TRUE,
                     TRUE, 5);
  box2 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 5);
  entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry), (guint16)PATH_MAX);
  gtk_signal_connect(GTK_OBJECT(entry), "activate",
                     GTK_SIGNAL_FUNC(activate_entry), NULL);
  gtk_box_pack_start(GTK_BOX(box2), entry, TRUE, TRUE, 5);
  scroll = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                               GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  frame = gtk_frame_new("Directories.");
  gtk_box_pack_start(GTK_BOX(box2), frame, TRUE, TRUE, 5);
  gtk_container_add(GTK_CONTAINER(frame), scroll);
  dclist = gtk_clist_new(1);
  gtk_signal_connect(GTK_OBJECT(dclist), "select-row",
                     GTK_SIGNAL_FUNC(select_row), dclist);
  gtk_widget_set_usize(dclist, 250, 100);
  gtk_clist_set_auto_sort(GTK_CLIST(dclist), TRUE);
  gtk_container_add(GTK_CONTAINER(scroll), dclist);
  scroll = gtk_scrolled_window_new(NULL, NULL);
  frame = gtk_frame_new("Files.");
  gtk_box_pack_start(GTK_BOX(box2), frame, TRUE, TRUE, 5);
  gtk_container_add(GTK_CONTAINER(frame), scroll);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                               GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  fclist = gtk_clist_new(1);
  gtk_signal_connect(GTK_OBJECT(fclist), "select-row",
                     GTK_SIGNAL_FUNC(select_row), fclist);
  gtk_widget_set_usize(fclist, 250, 100);
  gtk_clist_set_auto_sort(GTK_CLIST(fclist), TRUE);
  gtk_container_add(GTK_CONTAINER(scroll), fclist);
  box2 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 5);
  box3 = gtk_hbox_new(FALSE, 0);
  button = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(add_files_or_directories), NULL);
  gtk_box_pack_start(GTK_BOX(box2), button, TRUE, FALSE, 5);
  gtk_container_add(GTK_CONTAINER(button), box3);
  arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
  gtk_box_pack_start(GTK_BOX(box3), arrow, TRUE, FALSE, 2);
  label = gtk_label_new("Add");
  gtk_box_pack_start(GTK_BOX(box3), label, TRUE, FALSE, 2);
  box3 = gtk_hbox_new(FALSE, 0);
  button = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(remove_files_or_directories), NULL);
  gtk_box_pack_start(GTK_BOX(box2), button, TRUE, FALSE, 5);
  gtk_container_add(GTK_CONTAINER(button), box3);
  arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_NONE);
  gtk_box_pack_start(GTK_BOX(box3), arrow, TRUE, FALSE, 2);
  label = gtk_label_new("Remove");
  gtk_box_pack_start(GTK_BOX(box3), label, TRUE, FALSE, 2);
  pbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box2), pbox, TRUE, TRUE, 5);
  preview = gnome_pixmap_new_from_file("");
  gtk_box_pack_start(GTK_BOX(pbox), preview, TRUE, TRUE, 5);
  box2 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 5);
  adclist = gtk_clist_new(1);
  gtk_widget_set_usize(adclist, 250, 100);
  scroll = gtk_scrolled_window_new(NULL, NULL);
  frame = gtk_frame_new("Current Directory Selection.");
  gtk_box_pack_start(GTK_BOX(box2), frame, TRUE, TRUE, 5);
  gtk_container_add(GTK_CONTAINER(frame), scroll);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                               GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scroll), adclist);
  gtk_signal_connect(GTK_OBJECT(adclist), "select-row",
                     GTK_SIGNAL_FUNC(select_row), adclist);
  dtemp = g_list_copy(dirnames);
  for (li = g_list_nth(dirnames, 1); li != NULL; li = g_list_next(li)) {
    name[0] = li->data;
    gtk_clist_append(GTK_CLIST(adclist), name);
  }
  afclist = gtk_clist_new(1);
  gtk_widget_set_usize(afclist, 250, 100);
  scroll = gtk_scrolled_window_new(NULL, NULL);
  frame = gtk_frame_new("Current File Selection.");
  gtk_box_pack_start(GTK_BOX(box2), frame, TRUE, TRUE, 5);
  gtk_container_add(GTK_CONTAINER(frame), scroll);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                               GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scroll), afclist);
  gtk_signal_connect(GTK_OBJECT(afclist), "select-row",
                     GTK_SIGNAL_FUNC(select_row), afclist);
  ftemp = g_list_copy(indfiles);
  for (li = g_list_first(indfiles); li != NULL; li = g_list_next(li)) {
    name[0] = li->data;
    gtk_clist_append(GTK_CLIST(afclist), name);
  }
  get_dir_info(g_get_home_dir());
  gtk_widget_show_all(filesel);
}
